#!/usr/local/bin/tclsh
#
# test-pcm.tcl
#
# Script to test the sndfpcm functions by playing with combinations of 
# the environment and command-line options.
#
# 1997may16 dpwe@icsi.berkeley.edu
# $Header: /u/drspeech/src/dpwelib/testsnds/RCS/test-pcm.tcl,v 1.7 1997/06/21 00:14:13 dpwe Exp $

if {[llength $argv] > 0} {
    set cmddir [lindex $argv 0]
    set datadir [file dirname $argv0]
} else {
    puts stderr "$argv0: no directory given to find sndrfmt; assuming \".\""
    set cmddir  "."
    set datadir "../testsnds"
}
set sndrfmt "$cmddir/sndrfmt"
set sndcmp  "$cmddir/sndcmp"
set tmpsnd  "./tmpsnd"

set tstsnd    "$datadir/testR22C1Fs.beraw"
set tstsndle  "$datadir/testR22C1Fs.leraw"
set tstsndaif "$datadir/testR22C1Fs.aif"


proc ResultCode code {
    switch $code {
        0 {return TCL_OK}
        1 {return TCL_ERROR}
        2 {return TCL_RETURN}
        3 {return TCL_BREAK}
        4 {return TCL_CONTINUE}
    }
    return "Invalid result code $code"
}

proc OutputTestError {id command expectCode expectResult resultCode result} {
    puts stderr "======== Test $id failed ========"
    puts stderr $command
    puts stderr "==== Result was: [ResultCode $resultCode]:"
    puts stderr $result
    puts stderr "==== Expected : [ResultCode $expectCode]:"
    puts stderr $expectResult
    puts stderr "===="
}

# Test Procedure used by all tests
# id is the test identifier
# code is the test scenario
# optional -dontclean argument will stop the test classes being cleaned out
# alternatively -cleanup {script} will execute script before cleaning out

proc Test {id command expectCode expectResult args} {
    # debug
    puts "------------------------- Test $id:"

    set resultCode [catch {uplevel $command} result]

    if {($resultCode != $expectCode) ||
        ([string compare $result $expectResult] != 0)} {
        OutputTestError $id $command $expectCode $expectResult $resultCode \
                $result
    }

   if {[llength $args] == 0 || [lindex $args 0] != "-dontclean"} {
      if { [llength $args] > 0 && [lindex $args 0] == "-cleanup" } {
	  # post-completion cleanup specified
	  if { [llength $args] != 2 }  {
	      puts stderr "Test: -cleanup specified without cleanup script"
          } else {
	      set cscrpt [lindex $args 1]
 	      if {[catch {uplevel $cscrpt} rslt]} {
	          puts stderr "**Cleanup string \"$cscrpt\" returned \"$rslt\""
	      }
	  }
      }
   }
}

global results

Test 1.1 {
    # Standard env string version
    set env(PCMFORMAT) R22050C1Fs
    catch {exec $sndrfmt -S PCM -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 22050 Hz	 Duration  1.016 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 1.2 {
    # sampling rate in env string
    set env(PCMFORMAT) C1R16000Fs
    catch {exec $sndrfmt -S PCM -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 16000 Hz	 Duration  1.400 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 1.3 {
    # channels in env string
    set env(PCMFORMAT) C2R16000Fs
    catch {exec $sndrfmt -S PCM -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 16000 Hz	 Duration  0.700 s
  Stereo 	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 1.4 {
    # format in env string
    set env(PCMFORMAT) C2R16000Fc
    catch {exec $sndrfmt -S PCM -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 16000 Hz	 Duration  1.400 s
  Stereo 	 	 linear bytes
  data bytes 44812 	 info bytes      4}

Test 2.1 {
    # no PCMFORMAT string
    unset env(PCMFORMAT)
    catch {exec $sndrfmt -S PCM -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 22050 Hz	 Duration  1.016 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 2.2 {
    # rate alone
    set env(PCMFORMAT)   R16000
    catch {exec $sndrfmt -S PCM -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 16000 Hz	 Duration  1.400 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 2.3 {
    # chans alone
    set env(PCMFORMAT)   C2
    catch {exec $sndrfmt -S PCM -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 22050 Hz	 Duration  0.508 s
  Stereo 	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 2.4 {
    # format alone
    set env(PCMFORMAT)   Fl
    catch {exec $sndrfmt -S PCM -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 22050 Hz	 Duration  0.508 s
  Mono   	 	 32 bit linear
  data bytes 44812 	 info bytes      4}

Test 3.1 {
    # Header skip (Xn)
    set env(PCMFORMAT)	R16000C1X320Fs
    catch {exec $sndrfmt -S PCM -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 16000 Hz	 Duration  1.390 s
  Mono   	 	 16 bit linear
  data bytes 44492 	 info bytes      4}

Test 4.1 {
    # Format string after format specifier (1997may16)
    # - test first without
    set env(PCMFORMAT)	R16000C1Fs
    catch {exec $sndrfmt -S PCM -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 16000 Hz	 Duration  1.400 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 4.2 {
    # Format string after format specifier (1997may16)
    # - compare to with
    set env(PCMFORMAT)	R16000C1Fs
    catch {exec $sndrfmt -S PCM/R8000C2Fs -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate  8000 Hz	 Duration  1.400 s
  Stereo 	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 5.1 {
    # Format key need not come last, right?
    # Also, it accepts "8" and "16"
    catch {exec $sndrfmt -S PCM/F16R8000C2 -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate  8000 Hz	 Duration  1.400 s
  Stereo 	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 5.2 {
    # Format key in odd place/ format as number of bits/ rate in kHz
    catch {exec $sndrfmt -S PCM/R22F8C1 -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 22050 Hz	 Duration  2.032 s
  Mono   	 	 linear bytes
  data bytes 44812 	 info bytes      4}

Test 6.1 {
    # Test endian specifier
    catch {exec $sndcmp -S PCM/C1FsEb $tstsnd 2> /dev/null} rslt
    catch {exec $sndcmp -S PCM/C1FsEl $tstsnd 2> /dev/null} rslt2
    append rslt "\n$rslt2"
} 0 {Channel 0:
Stats for Sound1 (22406 pts):mean = 250.000;  sd =  1856.41;  rms =  1873.17
Channel 0:
Stats for Sound1 (22406 pts):mean = -74.496;  sd = 18714.35;  rms = 18714.50}

Test 6.2 {
    # Test little-endian conversion
    exec $sndrfmt -S PCM/R22C1FsEl $tstsndle -T AIFF $tmpsnd
    exec cmp -l $tmpsnd $tstsndaif
} 0 {}

Test 6.3 {
    # Big-to-little
    exec $sndrfmt -S PCM/R22C1FsEb -T PCM/R22C1FsEl $tstsnd $tmpsnd
    exec cmp $tmpsnd $tstsndle
} 0 {}

Test 6.4 {
    # Little-to-big
    exec $sndrfmt -S PCM/R22C1FsEl -T PCM/R22C1FsEb $tstsndle $tmpsnd
    exec cmp $tmpsnd $tstsnd
} 0 {}

# Had some problems with the environment strings:
# Actually, SNDFFTYPE appears to make PCMFORMAT redundant
# (not quite)
# but want to check their effects anyway

Test 7.1 {
    # Neither env set
    catch {unset env(PCMFORMAT)}
    catch {unset env(SNDFFTYPE)}
    catch {exec $sndrfmt -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 22050 Hz	 Duration  1.016 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 7.2 {
    # Just PCMFORMAT set
    set env(PCMFORMAT)   "R8C2Fc"
    catch {unset env(SNDFFTYPE)}
    catch {exec $sndrfmt -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate  8000 Hz	 Duration  2.801 s
  Stereo 	 	 linear bytes
  data bytes 44812 	 info bytes      4}

Test 7.3 {
    # Just SNDFFTYPE set; no specifier
    catch {unset env(PCMFORMAT)}
    set env(SNDFFTYPE)   "PCM"
    catch {exec $sndrfmt -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 22050 Hz	 Duration  1.016 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 7.4 {
    # Just SNDFFTYPE set; empty specifier
    catch {unset env(PCMFORMAT)}
    set env(SNDFFTYPE)   "PCM/"
    catch {exec $sndrfmt -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 22050 Hz	 Duration  1.016 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 7.5 {
    # SNDFFTYPE set with specifier
    set env(SNDFFTYPE)   "PCM/R8C2Fc"
    catch {unset env(PCMFTYPE)}
    catch {exec $sndrfmt -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate  8000 Hz	 Duration  2.801 s
  Stereo 	 	 linear bytes
  data bytes 44812 	 info bytes      4}

Test 7.6 {
    # Both set - SNDFTYPE takes priority if has post-bit
    set env(SNDFFTYPE)	"PCM/R8C2Fc"
    set env(PCMFORMAT) 	"R16C1Fs"
    catch {exec $sndrfmt -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate  8000 Hz	 Duration  2.801 s
  Stereo 	 	 linear bytes
  data bytes 44812 	 info bytes      4}

Test 7.7 {
    # Both set - SNDFTYPE takes priority, but only for specified fields
    set env(SNDFFTYPE)	"PCM/"
    set env(PCMFORMAT) 	"R16C1Fs"
    catch {exec $sndrfmt -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 16000 Hz	 Duration  1.400 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 7.8 {
    # Both set - but no post-bit in SNDFFTYPE allows PCMFORMAT to rule
    set env(SNDFFTYPE)	"PCM"
    set env(PCMFORMAT) 	"R16C2Fs"
    catch {exec $sndrfmt -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 16000 Hz	 Duration  0.700 s
  Stereo 	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 7.9 {
    # Precedence of PCMFORMAT, SNDFFTYPE & command line - 
    #  PCMFORMAT has SNDFFTYPE overlayed
    set env(PCMFORMAT) 	"R16C1Fc"
    set env(SNDFFTYPE)	"PCM/R8C2"
    catch {exec $sndrfmt -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate  8000 Hz	 Duration  2.801 s
  Stereo 	 	 linear bytes
  data bytes 44812 	 info bytes      4}

Test 7.10 {
    # Precedence of PCMFORMAT, SNDFFTYPE & command line - 
    #  command line used instead of SNDFFTYPE if both specified
    set env(PCMFORMAT) 	"R16C1Fc"
    set env(SNDFFTYPE)	"PCM/R8C2"
    catch {exec $sndrfmt -S PCM/R32 -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.beraw : (PCM)
  Sample rate 32000 Hz	 Duration  1.400 s
  Mono   	 	 linear bytes
  data bytes 44812 	 info bytes      4}
    

Test 8.1 {
    # Abbot formatting
    catch {unset env(PCMFORMAT)}
    catch {unset env(SNDFFTYPE)}
    exec $sndrfmt -S PCM -T PCM/Abb $tstsnd $tmpsnd
    # See that it added two bytes on the end (from data bytes)
    catch {exec $sndrfmt -S PCM -q $tmpsnd} rslt
    # See that they are ignored if we specify it's abbot
    catch {exec $sndrfmt -S PCM/Abb -q $tmpsnd} rslt2
    append rslt "\n$rslt2"
    set rslt
} 0 {./tmpsnd : (PCM)
  Sample rate 22050 Hz	 Duration  1.016 s
  Mono   	 	 16 bit linear
  data bytes 44814 	 info bytes      4
./tmpsnd : (PCM)
  Sample rate 22050 Hz	 Duration  1.016 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 8.2 {
    # Check Abbot clipping
    exec $sndrfmt -S PCM -T PCM -g 10 $tstsnd $tmpsnd
    catch {exec $sndcmp -S PCM $tmpsnd} rslt
    # Writing Abbot for this very clipped sound makes the RMS smaller
    # and mean higher...
    exec $sndrfmt -S PCM -T PCM/Abb -g 10 $tstsnd $tmpsnd
    catch {exec $sndcmp -S PCM/Abb $tmpsnd} rslt2
    append rslt "\n$rslt2"
    # But if you don't read it as Abbit, the large final val changes...
    catch {exec $sndcmp -S PCM $tmpsnd} rslt2
    append rslt "\n$rslt2"
    set rslt
    # Note that stderr output comes after stdout output in results
} 0 {Channel 0:
Stats for Sound1 (22406 pts):mean = 2485.725;  sd = 16681.65;  rms = 16865.83
22406 frames to process.  Each dot is 8192 frames:
...
Channel 0:
Stats for Sound1 (22406 pts):mean = 2485.765;  sd = 16681.56;  rms = 16865.75
22406 frames to process.  Each dot is 8192 frames:
...
Channel 0:
Stats for Sound1 (22407 pts):mean = 2484.191;  sd = 16682.85;  rms = 16866.80
22407 frames to process.  Each dot is 8192 frames:
...}

Test 8.3 {
    # Check that abbot read of a stream ends on 0x8000
    exec $sndrfmt -S PCM -T PCM/Abb $tstsnd $tmpsnd~
    exec cat $tmpsnd~ | $sndrfmt -S PCM/Abb - $tmpsnd
    catch {exec $sndrfmt -S PCM/Abb -q $tmpsnd} rslt
    # .. but reading the same stream without knowing it is Abb adds 2 bytes
    exec cat $tmpsnd~ | $sndrfmt -S PCM - $tmpsnd
    catch {exec $sndrfmt -S PCM -q $tmpsnd} rslt2
    append rslt "\n$rslt2"
    # .. but reading a non-Abb stream from a pipe as an Abb *can't*
    # lose the last two bytes - EOF supplants 0x8000
    exec $sndrfmt -S PCM -T PCM $tstsnd $tmpsnd~
    exec cat $tmpsnd~ | $sndrfmt -S PCM/Abb - $tmpsnd
    catch {exec $sndrfmt -S PCM/Abb -q $tmpsnd} rslt2
    append rslt "\n$rslt2"
    #
    exec cat $tmpsnd~ | $sndrfmt -S PCM - $tmpsnd
    catch {exec $sndrfmt -S PCM -q $tmpsnd} rslt2
    append rslt "\n$rslt2"
    #
    set rslt
} 0 {./tmpsnd : (PCM)
  Sample rate 22050 Hz	 Duration  1.016 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4
./tmpsnd : (PCM)
  Sample rate 22050 Hz	 Duration  1.016 s
  Mono   	 	 16 bit linear
  data bytes 44814 	 info bytes      4
./tmpsnd : (PCM)
  Sample rate 22050 Hz	 Duration  1.016 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4
./tmpsnd : (PCM)
  Sample rate 22050 Hz	 Duration  1.016 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4}



# Test:
#  Future:  end abbot stream on 0x8000
