#!/usr/local/bin/tclsh
#
# test-sndrfmt.tcl
#
# Script to run a bunch of test sounds through sndrfmt 
# to check that it still does what we thought it would
# after changes.
#
# 1997mar07 dpwe@icsi.berkeley.edu
# $Header: /u/drspeech/src/dpwelib/testsnds/RCS/test-sndrfmt.tcl,v 1.4 1997/05/24 03:04:47 dpwe Exp $

if {$argc > 1} {
    set sndrfmt "[lindex $argv 0]/sndrfmt"
    set datadir [file dirname $argv0]
} else {
    puts stderr "$argv0: no directory given to find sndrfmt; assuming \".\""
    set sndrfmt "./sndrfmt"
    set datadir "../testsnds"
}

proc ResultCode code {
    switch $code {
        0 {return TCL_OK}
        1 {return TCL_ERROR}
        2 {return TCL_RETURN}
        3 {return TCL_BREAK}
        4 {return TCL_CONTINUE}
    }
    return "Invalid result code $code"
}

proc OutputTestError {id command expectCode expectResult resultCode result} {
    puts stderr "======== Test $id failed ========"
    puts stderr $command
    puts stderr "==== Result was: [ResultCode $resultCode]:"
    puts stderr $result
    puts stderr "==== Expected : [ResultCode $expectCode]:"
    puts stderr $expectResult
    puts stderr "===="
}

# Test Procedure used by all tests
# id is the test identifier
# code is the test scenario
# optional -dontclean argument will stop the test classes being cleaned out
# alternatively -cleanup {script} will execute script before cleaning out

proc Test {id command expectCode expectResult args} {
    # debug
    puts "------------------------- Test $id:"

    set resultCode [catch {uplevel $command} result]

    if {($resultCode != $expectCode) ||
        ([string compare $result $expectResult] != 0)} {
        OutputTestError $id $command $expectCode $expectResult $resultCode \
                $result
    }

   if {[llength $args] == 0 || [lindex $args 0] != "-dontclean"} {
      if { [llength $args] > 0 && [lindex $args 0] == "-cleanup" } {
	  # post-completion cleanup specified
	  if { [llength $args] != 2 }  {
	      puts stderr "Test: -cleanup specified without cleanup script"
          } else {
	      set cscrpt [lindex $args 1]
 	      if {[catch {uplevel $cscrpt} rslt]} {
	          puts stderr "**Cleanup string \"$cscrpt\" returned \"$rslt\""
	      }
	  }
      }
   }
}

global results

set tmpsnd "./tmpsnd"
set tstsnd "$datadir/testR22C1Fs.aif"

Test 1.1 {
    # -q (query) fn
    catch {exec $sndrfmt -q $tstsnd} rslt
    set rslt
} 0 {../testsnds/testR22C1Fs.aif : (AIFF)
  Sample rate 22050 Hz	 Duration  1.016 s
  Mono   	 	 16 bit linear
  data bytes 44812 	 info bytes      4}

Test 1.2 {
    # Write AIFF
    exec rm -f $tmpsnd
    exec $sndrfmt -T AIFF $tstsnd $tmpsnd
    exec cmp $datadir/testR22C1Fs.aif $tmpsnd
} 0 {}

Test 1.3 {
    # Write MSWAVE
    exec rm -f $tmpsnd
    exec $sndrfmt -T MSWAVE $tstsnd $tmpsnd
    exec cmp $datadir/testR22C1Fs.wav $tmpsnd
} 0 {}

Test 1.4 {
    # Write NeXT
    exec rm -f $tmpsnd
    exec $sndrfmt -T NeXT $tstsnd $tmpsnd
    exec cmp $datadir/testR22C1Fs.au $tmpsnd
} 0 {}

Test 1.5 {
    # Write NIST
    exec rm -f $tmpsnd
    exec $sndrfmt -T NIST $tstsnd $tmpsnd
    exec cmp $datadir/testR22C1Fs.nist $tmpsnd
} 0 {}

Test 1.6 {
    # Write IRCAM
    exec rm -f $tmpsnd
    exec $sndrfmt -T IRCAM $tstsnd $tmpsnd
    exec cmp $datadir/testR22C1Fs.irc $tmpsnd
} 0 {}

Test 1.7 {
    # Write PCM
    exec rm -f $tmpsnd
    exec $sndrfmt -T PCM $tstsnd $tmpsnd
    exec cmp $datadir/testR22C1Fs.beraw $tmpsnd
} 0 {}

Test 2.1 {
    # Strip header (same as write PCM)
    exec rm -f $tmpsnd
    exec $sndrfmt -s $tstsnd $tmpsnd
    exec cmp $datadir/testR22C1Fs.beraw $tmpsnd
} 0 {}

Test 2.2 {
    # Add header
    exec rm -f $tmpsnd
    exec $sndrfmt -T AIFF -a -r 22050 -c 1 -f s $datadir/testR22C1Fs.beraw $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 3.1 {
    # Read each file type with identify - AIF
    exec rm -f $tmpsnd
    exec $sndrfmt -T AIFF $datadir/testR22C1Fs.aif $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 3.2 {
    # NeXT
    exec rm -f $tmpsnd
    exec $sndrfmt -T AIFF $datadir/testR22C1Fs.au $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 3.3 {
    # IRCAM
    exec rm -f $tmpsnd
    exec $sndrfmt -T AIFF $datadir/testR22C1Fs.irc $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 3.4 {
    # NIST
    exec rm -f $tmpsnd
    exec $sndrfmt -T AIFF $datadir/testR22C1Fs.nist $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 3.5 {
    # MSWAVE
    exec rm -f $tmpsnd
    exec $sndrfmt -T AIFF $datadir/testR22C1Fs.wav $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 4.1 {
    # Read each file type with type specified - AIF
    exec rm -f $tmpsnd
    exec $sndrfmt -S AIFF -T AIFF $datadir/testR22C1Fs.aif $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 4.2 {
    # NeXT
    exec rm -f $tmpsnd
    exec $sndrfmt -S NeXT -T AIFF $datadir/testR22C1Fs.au $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 4.3 {
    # IRCAM
    exec rm -f $tmpsnd
    exec $sndrfmt -S IRCAM -T AIFF $datadir/testR22C1Fs.irc $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 4.4 {
    # NIST
    exec rm -f $tmpsnd
    exec $sndrfmt -S NIST -T AIFF $datadir/testR22C1Fs.nist $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 4.5 {
    # MSWAVE
    exec rm -f $tmpsnd
    exec $sndrfmt -S MSWAVE -T AIFF $datadir/testR22C1Fs.wav $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 4.6 {
    # ESPS
    exec rm -f $tmpsnd
    exec $sndrfmt -S ESPS -T AIFF $datadir/testR22C1Fs.sd $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 4.7 {
    # PCM
    exec rm -f $tmpsnd
    exec echo "PCMFORMAT=R22050C1Fs; export PCMFORMAT; $sndrfmt -S PCM -T AIFF $datadir/testR22C1Fs.beraw $tmpsnd" | sh
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 5.1 {
    # Test just changing header fields
    exec rm -f $tmpsnd
    exec $sndrfmt -h -r 12345 -c 2 -f l $tstsnd $tmpsnd
    catch {exec $sndrfmt -q $tmpsnd} rslt
    # .. but check that the data bit stayed the same
    exec mv $tmpsnd $tmpsnd~
    exec $sndrfmt -s $tmpsnd~ $tmpsnd
    catch {exec cmp $tmpsnd $datadir/testR22C1Fs.beraw} r2
    exec rm $tmpsnd~
    # tmpsnd is forced to be an integral number of 8 byte sample frames, 
    # so is actually only 44808 bytes, not 44812, hence EOF
    append rslt "\n$r2"
} 0 {./tmpsnd : (AIFF)
  Sample rate 12345 Hz	 Duration  0.454 s
  Stereo 	 	 32 bit linear
  data bytes 44812 	 info bytes      4
cmp: EOF on ./tmpsnd}

Test 6.1 {
    # stdin if infname == "-"
    exec rm -f $tmpsnd
    exec cat $tstsnd | $sndrfmt -T AIFF - $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 6.2 {
    # stdout if outfname == "-"
    exec rm -f $tmpsnd
    exec $sndrfmt -T AIFF $tstsnd - > $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

Test 6.3 {
    # stdin to stdout
    exec rm -f $tmpsnd
    exec cat $tstsnd | $sndrfmt -T AIFF - - > $tmpsnd
    exec cmp $tstsnd $tmpsnd
} 0 {}

